@@ -1,4 +1,4 @@
-# encoding=UTF8
+# coding=utf-8
 
 # Copyright (c) 2012 Rackspace Hosting
 # All Rights Reserved.
@@ -17,20 +17,17 @@
 
 """Unit tests for SQLAlchemy specific code."""
 import logging
-from oslo.config import cfg
 
-import _mysql_exceptions
 import fixtures
 import mock
+from oslo.config import cfg
 from oslotest import base as oslo_test
 import sqlalchemy
-from sqlalchemy import Column, MetaData, Table, UniqueConstraint
-from sqlalchemy import DateTime, Integer, String
-from sqlalchemy import exc as sqla_exc
-from sqlalchemy.exc import DataError
+from sqlalchemy import Column, MetaData, Table
+from sqlalchemy import Integer, String
 from sqlalchemy.ext.declarative import declarative_base
 
-from oslo.db import exception as db_exc
+from oslo.db import exception
 from oslo.db import options as db_options
 from oslo.db.sqlalchemy import models
 from oslo.db.sqlalchemy import session
@@ -40,90 +37,6 @@ from oslo.db.sqlalchemy import test_base
 BASE = declarative_base()
 _TABLE_NAME = '__tmp__test__tmp__'
 
-
-class TmpTable(BASE, models.ModelBase):
-    __tablename__ = _TABLE_NAME
-    id = Column(Integer, primary_key=True)
-    foo = Column(Integer)
-
-
-class SessionErrorWrapperTestCase(test_base.DbTestCase):
-    def setUp(self):
-        super(SessionErrorWrapperTestCase, self).setUp()
-        meta = MetaData()
-        meta.bind = self.engine
-        test_table = Table(_TABLE_NAME, meta,
-                           Column('id', Integer, primary_key=True,
-                                  nullable=False),
-                           Column('deleted', Integer, default=0),
-                           Column('deleted_at', DateTime),
-                           Column('updated_at', DateTime),
-                           Column('created_at', DateTime),
-                           Column('foo', Integer),
-                           UniqueConstraint('foo', name='uniq_foo'))
-        test_table.create()
-        self.addCleanup(test_table.drop)
-
-    def test_flush_wrapper(self):
-        _session = self.sessionmaker()
-
-        tbl = TmpTable()
-        tbl.update({'foo': 10})
-        tbl.save(_session)
-
-        tbl2 = TmpTable()
-        tbl2.update({'foo': 10})
-        self.assertRaises(db_exc.DBDuplicateEntry, tbl2.save, _session)
-
-    def test_execute_wrapper(self):
-        _session = self.sessionmaker()
-        with _session.begin():
-            for i in [10, 20]:
-                tbl = TmpTable()
-                tbl.update({'foo': i})
-                tbl.save(session=_session)
-
-            method = _session.query(TmpTable).\
-                filter_by(foo=10).\
-                update
-            self.assertRaises(db_exc.DBDuplicateEntry,
-                              method, {'foo': 20})
-
-    def test_ibm_db_sa_raise_if_duplicate_entry_error_duplicate(self):
-        # Tests that the session._raise_if_duplicate_entry_error method
-        # translates the duplicate entry integrity error for the DB2 engine.
-        statement = ('INSERT INTO key_pairs (created_at, updated_at, '
-                     'deleted_at, deleted, name, user_id, fingerprint) VALUES '
-                     '(?, ?, ?, ?, ?, ?, ?)')
-        params = ['20130918001123627099', None, None, 0, 'keypair-23474772',
-                  '974a7c9ffde6419f9811fcf94a917f47',
-                  '7d:2c:58:7f:97:66:14:3f:27:c7:09:3c:26:95:66:4d']
-        orig = sqla_exc.SQLAlchemyError(
-            'SQL0803N  One or more values in the INSERT statement, UPDATE '
-            'statement, or foreign key update caused by a DELETE statement are'
-            ' not valid because the primary key, unique constraint or unique '
-            'index identified by "2" constrains table "NOVA.KEY_PAIRS" from '
-            'having duplicate values for the index key.')
-        integrity_error = sqla_exc.IntegrityError(statement, params, orig)
-        self.assertRaises(db_exc.DBDuplicateEntry,
-                          session._raise_if_duplicate_entry_error,
-                          integrity_error, 'ibm_db_sa')
-
-    def test_ibm_db_sa_raise_if_duplicate_entry_error_no_match(self):
-        # Tests that the session._raise_if_duplicate_entry_error method
-        # does not raise a DBDuplicateEntry exception when it's not a matching
-        # integrity error.
-        statement = ('ALTER TABLE instance_types ADD CONSTRAINT '
-                     'uniq_name_x_deleted UNIQUE (name, deleted)')
-        params = None
-        orig = sqla_exc.SQLAlchemyError(
-            'SQL0542N  The column named "NAME" cannot be a column of a '
-            'primary key or unique key constraint because it can contain null '
-            'values.')
-        integrity_error = sqla_exc.IntegrityError(statement, params, orig)
-        session._raise_if_duplicate_entry_error(integrity_error, 'ibm_db_sa')
-
-
 _REGEXP_TABLE_NAME = _TABLE_NAME + "regexp"
 
 
@@ -213,53 +126,6 @@ class FakeDB2Engine(object):
         pass
 
 
-class TestDBDisconnected(oslo_test.BaseTestCase):
-
-    def _test_ping_listener_disconnected(self, connection):
-        engine_args = {
-            'pool_recycle': 3600,
-            'echo': False,
-            'convert_unicode': True}
-
-        engine = sqlalchemy.create_engine(connection, **engine_args)
-        with mock.patch.object(engine, 'dispose') as dispose_mock:
-            self.assertRaises(sqlalchemy.exc.DisconnectionError,
-                              session._ping_listener, engine,
-                              FakeDBAPIConnection(), FakeConnectionRec(),
-                              FakeConnectionProxy())
-            dispose_mock.assert_called_once_with()
-
-    def test_mysql_ping_listener_disconnected(self):
-        def fake_execute(sql):
-            raise _mysql_exceptions.OperationalError(self.mysql_error,
-                                                     ('MySQL server has '
-                                                      'gone away'))
-        with mock.patch.object(FakeCursor, 'execute',
-                               side_effect=fake_execute):
-            connection = 'mysql://root:password@fakehost/fakedb?charset=utf8'
-            for code in [2006, 2013, 2014, 2045, 2055]:
-                self.mysql_error = code
-                self._test_ping_listener_disconnected(connection)
-
-    def test_db2_ping_listener_disconnected(self):
-
-        def fake_execute(sql):
-            raise OperationalError('SQL30081N: DB2 Server '
-                                   'connection is no longer active')
-        with mock.patch.object(FakeCursor, 'execute',
-                               side_effect=fake_execute):
-            # TODO(dperaza): Need a fake engine for db2 since ibm_db_sa is not
-            # in global requirements. Change this code to use real IBM db2
-            # engine as soon as ibm_db_sa is included in global-requirements
-            # under openstack/requirements project.
-            fake_create_engine = lambda *args, **kargs: FakeDB2Engine()
-            with mock.patch.object(sqlalchemy, 'create_engine',
-                                   side_effect=fake_create_engine):
-                connection = ('ibm_db_sa://db2inst1:openstack@fakehost:50000'
-                              '/fakedab')
-                self._test_ping_listener_disconnected(connection)
-
-
 class MySQLModeTestCase(test_base.MySQLOpportunisticTestCase):
 
     def __init__(self, *args, **kwargs):
@@ -311,12 +177,13 @@ class MySQLStrictAllTablesModeTestCase(MySQLModeTestCase):
         value = 'a' * 512
         # String is too long.
         # With STRICT_ALL_TABLES or TRADITIONAL mode set, this is an error.
-        self.assertRaises(DataError,
+        self.assertRaises(exception.DBError,
                           self._test_string_too_long, value)
 
 
 class MySQLTraditionalModeTestCase(MySQLStrictAllTablesModeTestCase):
     """Test data integrity enforcement in MySQL TRADITIONAL mode.
+
     Since TRADITIONAL includes STRICT_ALL_TABLES, this inherits all
     STRICT_ALL_TABLES mode tests.
     """
@@ -585,3 +452,19 @@ class MysqlSetCallbackTest(oslo_test.BaseTestCase):
             "SELECT * FROM bar",
         ]
         self.assertEqual(exp_calls, engine._execs)
+
+
+class PatchStacktraceTest(test_base.DbTestCase):
+
+    def test_trace(self):
+        engine = self.engine
+        engine.connect()
+        with mock.patch.object(engine.dialect, "do_execute") as mock_exec:
+            session._add_trace_comments(engine)
+
+            engine.execute("select * from table")
+
+        call = mock_exec.mock_calls[0]
+
+        # we're the caller, see that we're in there
+        self.assertTrue("tests/sqlalchemy/test_sqlalchemy.py" in call[1][1])
